from timemachines.skaters.tsa.tsaconstant import TSA_CONSTANT_SKATERS
from timemachines.skaters.tsa.tsaensembles import TSA_DO_ENSEMBLE_SKATERS
from timemachines.skaters.tsa.tsahypocratic import TSA_DO_HYPOCRATIC_SKATERS

TSA_SKATERS = TSA_CONSTANT_SKATERS + TSA_DO_ENSEMBLE_SKATERS + TSA_DO_HYPOCRATIC_SKATERS